' Represents a series of points to draw.
<Serializable()> _
Public Class Polyline
    ' The points we draw.
    Public Points() As Point = {}

    ' Add a new point to the last point array.
    ' Don't add duplicates. Return True if we add a point.
    Public Function AddPoint(ByVal x As Integer, ByVal y As Integer) As Boolean
        ' See if this point is different from the previous one.
        If (Points.Length > 0) Then
            With Points(Points.Length - 1)
                If (x = .X) AndAlso (y = .Y) Then Return False
            End With
        End If

        ' Add the point.
        ReDim Preserve Points(0 To Points.Length)
        Points(Points.Length - 1) = New Point(x, y)
        Return True
    End Function

    ' Return True if we hold fewer than 2 points.
    Public Function IsEmpty() As Boolean
        Return Points.Length < 2
    End Function

    ' Draw.
    Public Sub Draw(ByVal gr As Graphics, ByVal the_pen As Pen)
        gr.DrawLines(the_pen, Points)
    End Sub

    ' Make a deep copy.
    Public Function Clone() As Polyline
        Return DirectCast(Me.MemberwiseClone, Polyline)
    End Function

    ' Return the points' bounds.
    Public Function Bounds() As Rectangle
        Dim xmin As Integer = Points(0).X
        Dim xmax As Integer = xmin
        Dim ymin As Integer = Points(0).Y
        Dim ymax As Integer = ymin
        For i As Integer = 0 To Points.Length - 1
            If xmin > Points(i).X Then xmin = Points(i).X
            If xmax < Points(i).X Then xmax = Points(i).X
            If ymin > Points(i).Y Then ymin = Points(i).Y
            If ymax < Points(i).Y Then ymax = Points(i).Y
        Next i

        Return New Rectangle(xmin, ymin, xmax - xmin + 1, ymax - ymin + 1)
    End Function
End Class
