Imports System.Drawing.Design
Imports System.ComponentModel

<ToolboxBitmap(GetType(ScribbleControl), "tbxScribble")> _
<Designer(GetType(ScribbleControlDesigner))> _
<EditorAttribute(GetType(ScribbleControlEditor), GetType(ComponentEditor))> _
Public Class ScribbleControl
    Inherits Control

    ' Raised after the user finishes a drawing series.
    Public Event DrawingChanged()

#Region "Public Properties"

    ' The points we draw.
    Private m_PolyPolyline As PolyPolyline = New PolyPolyline
    <TypeConverter(GetType(PolyPolylineConverter))> _
    <EditorAttribute(GetType(PolyPolylineEditor), GetType(UITypeEditor))> _
    Public Property PolyPolyline() As PolyPolyline
        Get
            Return m_PolyPolyline
        End Get
        Set(ByVal value As PolyPolyline)
            m_PolyPolyline = value
            Me.Invalidate()
        End Set
    End Property

    ' The color we use for the lines.
    ' The colors allowed for the lines.
    Public Enum LineColors
        Red
        Orange
        Yellow
        Green
        Blue
        Ingido
        Violet
        Black
        White
    End Enum
    Public Const NumLineColors As Integer = 9

    Private m_LineColor As LineColors = LineColors.Black
    <EditorAttribute(GetType(ColorIndexEditor), GetType(UITypeEditor))> _
    Public Property LineColor() As LineColors
        Get
            Return m_LineColor
        End Get
        Set(ByVal value As LineColors)
            m_LineColor = value
            Me.Invalidate()
        End Set
    End Property

    ' The thicknesses of the lines.
    Private m_LineWidth As Integer = 1
    <EditorAttribute(GetType(LineWidthEditor), GetType(UITypeEditor))> _
    Public Property LineWidth() As Integer
        Get
            Return m_LineWidth
        End Get
        Set(ByVal value As Integer)
            If value < 1 Then value = 1
            If value > 10 Then value = 10
            m_LineWidth = value
            Me.Invalidate()
        End Set
    End Property

#End Region ' Public Properties

    ' Use double buffering.
    Private Sub ScribbleControl_HandleCreated(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.HandleCreated
        Me.SetStyle( _
            ControlStyles.AllPaintingInWmPaint Or _
            ControlStyles.ResizeRedraw Or _
            ControlStyles.OptimizedDoubleBuffer, _
            True)
        Me.UpdateStyles()
    End Sub

    ' Draw the scribble.
    Private Sub ScribbleControl_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        ' Draw the background and a border.
        e.Graphics.Clear(Me.BackColor)
        ControlPaint.DrawBorder3D(e.Graphics, Me.ClientRectangle)

        ' Draw the scribble.
        Dim colors() As Color = {Color.Red, Color.Orange, Color.Yellow, Color.Green, Color.Blue, Color.Indigo, Color.Violet, Color.Black, Color.White}
        Dim line_color As Color = colors(m_LineColor)
        Using fg_pen As New Pen(line_color, m_LineWidth)
            m_PolyPolyline.Draw(e.Graphics, fg_pen)
        End Using
    End Sub

#Region "Drawing Routines"
    ' Start drawing.
    Private m_Drawing As Boolean = False
    Private Sub ScribbleControl_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If Not Enabled Then Exit Sub

        If e.Button = Windows.Forms.MouseButtons.Right Then
            ' Clear the PolyPolyline.
            m_PolyPolyline.Clear()
            Me.Invalidate()

            ' Let the program know, too.
            RaiseEvent DrawingChanged()
        Else
            ' Start a new Polyline.
            m_PolyPolyline.AddPolyline()

            ' Add the first point.
            m_PolyPolyline.AddPoint(e.X, e.Y)

            ' We're drawing.
            m_Drawing = True
        End If
    End Sub

    ' Continue drawing.
    Private Sub ScribbleControl_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If Not m_Drawing Then Exit Sub

        ' Add the new point and redraw.
        If m_PolyPolyline.AddPoint(e.X, e.Y) Then Me.Invalidate()
    End Sub

    ' Stop drawing.
    Private Sub ScribbleControl_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If Not m_Drawing Then Exit Sub
        m_Drawing = False

        ' Let the PolyPolyline know we are done with this polyline.
        m_PolyPolyline.EndPolyline()

        ' Let the program know, too.
        RaiseEvent DrawingChanged()
    End Sub
#End Region ' Drawing Routines

End Class
